// ASL sample code for Nuvoton's W836x7HG CIR device driver (ver. 1.8.2009.0427 or later)
// This ASL code assumes I/O base of the CIR device is 0x240 and IRQ3 is used.
//                       I/O base of the CIR wakeup device is 0x250.
//                       Extended Function Enable Register is at 0x4E
//
// H/W pins (W83677HG-i) used in this sample:
//
// pin 96   = CIRTX1
// pin  6   = CIRTX2
// pin 95   = CIRRX (for long-range IR receiver)
// pin 38   = CIRRXWB (for wide-band IR receiver)
// pin 13   = CIR LED (Open-drain, LOW for LED ON)
// pin 11   = Jack detection for CIR Tx emitter 1 (HIGH for Tx present)
// pin 12   = Jack detection for CIR Tx emitter 2 (HIGH for Tx present)
//
// Copyright (c) 2008 Nuvoton Technology Corp. All rights reserved.
//

OperationRegion(WIN2, SystemIO, 0x0000004E, 0x00000002)
Field(WIN2, ByteAcc, NoLock, Preserve)
{
    IND1, 0x00000008,
    DAT1, 0x00000008,   // Offset: 01h
}
IndexField(IND1, DAT1, ByteAcc, NoLock, Preserve)
{
    Offset(0x07),
    LDN1, 0x00000008,   // Index: 07h   ;logical device number
    Offset(0x27),
    GLO1, 0x00000008,   // Index: 27h   ;CIRWB, CIRRX, CIRTX1 
    Offset(0x2A),
    MFP1, 0x00000008,   // Index: 2Ah   ;CIRTX1 & CIRTX2 
    Offset(0x2F),
    SFR1, 0x00000008,   // Index: 2Fh
    
    // LDN = 0x06 and 0x0E
    Offset(0x30),
    ACT1, 0x00000008,   // Index: 30h   ;device active & inactive
    Offset(0x60),
    IOH1, 0x00000008,   // Index: 60h   ;CIR & CIR Wakeup base address high byte
    IOL1, 0x00000008,   // Index: 61h   ;CIR & CIR Wakeup base address low byte
    Offset(0x70),
    INT1, 0x00000008,   // Index: 70h   ;CIR IRQ

    // LDN = 0x0A
    offset(0xE0),
    VBAT, 0x00000008,   // Index: E0h   ;PSOUT# for CIR Wakeup
    
    // LDN = 0x0F    
    Offset(0xE4),
    PPO5, 0x00000008,   // Index: E4h   ;GPIO5 Push-Pull/OD select
    
    // LDN = 0x09
    Offset(0xEB),
    MSL5, 0x00000008,   // Index: EBh   ;GPIO 5 multi-function select register
    Offset(0xF4),
    IOR5, 0x00000008,   // Index: F4h   ;GPIO 5 IO setting
    Offset(0xF5),
    DAR5, 0x00000008,   // Index: F5h   ;GPIO 5 data register
}
Method(ENF1, 0x0, NotSerialized)
{
    Store(0x87, IND1)
    Store(0x87, IND1)
}

Method(EXF1, 0x0, NotSerialized)
{
    Store(0xAA, IND1)
}

Device(CIR)
{
    Name(_HID, EISAID("NTN0530"))           // Nuvoton's W836x7 CIR device
    Method(_STA, 0x00, NotSerialized)       
    {                                       
        Return(0x0F)                        // Assume the CIR device is always present
    }
    Method(_DIS, 0x0, NotSerialized)
    {
        ENF1()
        Store(0x06, LDN1)                   // Select LDN = 0x06
        Store(Zero, ACT1)                   // Inactive CIR device
        Store(0x0E, LDN1)                   // Select LDN = 0x0E
        Store(Zero, ACT1)                   // Inactive CIR Wakeup device
        EXF1()
    }
    Method(_CRS, 0x00, NotSerialized)
    {
        Name(BUF1, ResourceTemplate()
        {
            IO(Decode10, 0x0240, 0x0240, 0x00, 0x10)        // CIR Device: IOBase = 0x240
            IRQ(Edge, ActiveHigh, Shared){3}                // CIR Device: IRQ3
            IO(Decode10, 0x0250, 0x0250, 0x00, 0x10)        // CIR Wakeup Device: IOBase = 0x250
        })
        Return(BUF1)
    }
    Method(_SRS, 0x01, NotSerialized)
    {
        // Enabled the CIR and CIR Wakeup function and set the I/O resources.
        // Also configure the HW pins for CIR and CIR Wakeup device.
        //
        ENF1()
        
        // CIR Device
        Store(0x06, LDN1)                   // Select LDN = 0x06, CIR device
        CreateByteField(Arg0, 0x02, IOLA)   // IO base addr low byte
        CreateByteField(Arg0, 0x03, IOHA)   // IO base addr high byte
        CreateWordField(Arg0, 0x09, IRQE)   // IRQ enable bit
        Store(IOLA, IOL1)
        Store(IOHA, IOH1)
        if (IRQE)
        {
            FindSetRightBit(IRQE, Local0)
            Subtract(Local0, One, INT1)     // IRQ number
        }
        else
        {
            Store(Zero, INT1)               // No IRQ is specified
        }
        Store(One, ACT1)                    // Enable CIR device
        
        //Store( SFR1, Local0)
        //Or(Local0, 0x01, Local0)         // PORT80_EN = 1 for CIRWB pin
        //Or(Local0, 0x09, Local0)         // Intel bit enable for Intel chip
        //Store( Local0, SFR1)
        
        Store(GLO1, Local0)
        And(Local0, 0xBC, Local0)           // Clear bits 6, 1, 0
        Or(Local0, 0x41, Local0)            // bit6=CIRWB, bits1~0=CIRTX1 & CIRRX 
        Store(Local0, GLO1)                 
                                            
        Store(0x0F, LDN1)                   // Select LDN = F
        Store(PPO5, Local0)                 
        Or(Local0, 0x80, Local0)            
        Store(Local0, PPO5)                 // GP57 : Open drain
                                            
        Store(0x09, LDN1)                   // Select LDN = 9
        Store(ACT1, Local0)                 
        Or(Local0, 0x08, Local0)            
        Store(Local0, ACT1)                 // Activate GPIO5
        Store(MSL5, Local0)                 
        And(Local0, 0x1F, Local0)           
        Store(Local0, MSL5)                 // GP57, GP56, GP55 as GPIO pins
        Store(IOR5, Local0)                 
        And(Local0, 0x1F, Local0)           
        Or(Local0, 0x60, Local0)            
        Store(Local0, IOR5)                 // As GPO57, GPI56, GPI55
        Store(DAR5, Local0)                 
        Or(Local0, 0x80, Local0)            
        Store(Local0, DAR5)                 // Set GPO57 as HIGH, i.e. LED is off
        // CIR Device End
        
        // CIR Wakeup Device
        Store(0x0E, LDN1)                   // Select LDN = E, CIR wakeup device
        CreateByteField(Arg0, 0x0E, IOLB)   // IO base addr low byte
        CreateByteField(Arg0, 0x0F, IOHB)   // IO base addr high byte
        Store(IOLB, IOL1)
        Store(IOHB, IOH1)
        Store(One, ACT1)                    // Enable CIR Wakeup device
        
        Store(0x0A, LDN1)                   // Select LDN = A, ACPI device
        Store(0x08, VBAT)                   // Enable CIR wakeup function via PSOUT#
        
        //Store(GLO1, Local0)
        //And(Local0, 0xFC, Local0)         // Clear bits 1, 0
        //Or(Local0, 0x01, Local0)          // bits1~0 = 01, CIRTX1 & CIRRX 
        //Store(Local0, GLO1)
        // CIR Wakeup Device End
        
        EXF1()
    }
    Method(NCAP,0x00)
    {
        // Capability
        // Bit 31: Flash the LED based on CIR port driver requirement for Vista
        //         media center. 
        // Bit 30: Power on the LED when receiving the Rx data.  LED is controlled
        //         by the HW pin specified in the NRXL control method.
        // Bit 29: Offset of the Extended function enable register( 0: 2Eh, 1: 4Eh)
        // Bit 28: Auto detect dongle polarity. (Bit26 and Bit27 will not be effective when this bit enable)
        // Bit 27: IR Rx Invert Enable.
        //         0: Dongle Carrier ON is high, OFF/Idle is low.
        //         1: dongle Carrier ON is low, OFF/Idle is high.
        // Bit 26: Wide-band IR Rx Invert Enable.
        //         0: Dongle Carrier ON is high, OFF/Idle is low.
        //         1: dongle Carrier ON is low, OFF/Idle is high.
        // Bits 25-05: Reserved
        // Bits 04-00: Number of Tx emitters. At least 1, but 2 is required for Vista CIR DTM.
        
        Store(0x02, Local0)                 //Tx port number (1-31)
        Or(0xF0000000, Local0, Local0)      //mark as supporting the LED
        Return(Local0)
    }
    Method(NRXL,0x00)
    {
        // LED configuration for Rx.  This control method is needed if bit 30
        // of the return value of NCAP control method is 1.
        // 
        // GPO57 is used to controller the LED in this sample. And outputs LOW 
        // to power on the LED
        //
        Store(0x00, Local0)                // Bit 7 = 0, GPO57 = LOW is to power on the LED.
        ShiftLeft(Local0, 8, Local0)       
        Or(Local0, 0x80, Local0)           // Bit 7 = 1, GPO57 is used for the LED ON/OFF control
        ShiftLeft(Local0, 8, Local0)       
        Or(Local0, 0xF5, Local0)           // Offset of GPIO5 data register
        ShiftLeft(Local0, 8, Local0)       
        Or(Local0, 0x09, Local0)           // Logical Device Number = 0x09 
        
        Return(Local0) 
    }
    Method(NJKP,0x00)
    {
        // Tx emitter jack detection
        // Return the 32-bit value to indicate which the Tx emitters are present. (bits30-0)
        //
        ENF1()
        
        Store(0x09, LDN1)                   // Select LDN = 9
        Store(DAR5, Local0)                 // Read GP5 Data Register
        And(0x60, Local0, Local0)           // GP55 = for CIRTX1, GP56 = for CIRTX2
        ShiftRight(Local0, 0x05, Local0)    // Align to the bit 0
        
        EXF1()
        
        Return(Local0)
    }
    Method(NTXP,0x01)
    {
        // Select the Tx emitter
        // Input argument (32-bit): Tx port bitmask (bit30 ~ bit0)
        //
        ENF1()

        Store(MFP1, Local0)                 // Index 2Ah
        And(Local0, 0xE7, Local0)           // bit4 = CIRTX1, bit3 = CIRTX2
        And(Arg0, 0x03, Arg0)               // Consider the bitmask range as 0 ~ 3
        
        If( LEqual(Arg0, 0x00000001))
        {
            Or(Local0, 0x10, Local0)        // Enable CIRTX1
        }
        ElseIf( LEqual(Arg0, 0x00000002))
        {
            Or(Local0, 0x08, Local0)        // Enable CIRTX2
        }
        ElseIf( LEqual(Arg0, 0x00000003))
        {
            Or(Local0, 0x18, Local0)        // Enable CIRTX1, CIRTX2
        }
        
        Store(Local0, MFP1)
        
        EXF1()
    }
    Method(NLED,0x00)
    {
        // Flash the LED.  Optionally required for the CIR port driver for Vista 
        // media cetner.  This control method will be needed if bit 31 of the 
        // return value of NCAP control method is 1.
        //
        // GPO57 is used to controller the LED in this sample.
        //
        ENF1()
        
        Store(0x09, LDN1)                   // Select LDN = 9
        Store(DAR5, Local0)                 // Current GPIO5 state
        Store(0x0A, Local1)                 
        While(Local1)                       
        {                                   
            And(0x7F, Local0, DAR5)         // Set GPO57 state as LOW
            Sleep(0x32)                     // 10ms
            Or(0x80, Local0, DAR5)          // Set GPO57 state as HIGH
            Sleep(0x32)                     // 10ms
            Decrement(Local1)
        }
        
        EXF1()
    }
}
// End of CIR device and CIR Wakeup device.